#!/bin/bash

if [[ $EUID -ne 0 ]]; then
  whiptail \
    --title "Невозможно запустить nand-sata-install" \
    --msgbox "Утилита nand-sata-install работает только из под пользователя root!\n\nДля запуска утилиты необходимо перейти в профиль root или воспользоваться аналогичной командой: sudo nand-sata-install" \
    --ok-button "Ok"\
    12 70
  exit 1
fi

ROOT_UUID=$(sed -e 's/^.*root=//' -e 's/ .*$//' < /proc/cmdline)
ROOT_PARTITION=$(blkid | tr -d '":' | grep "${ROOT_UUID}" | awk '{print $1}')
ROOT_DEVICE=/dev/$(lsblk -ndo pkname ${ROOT_PARTITION})
EMMC_DEVICES=$(ls -d -1 /dev/mmcblk* 2>/dev/null | grep -w 'mmcblk[0-9]' | grep -v "${ROOT_DEVICE}")
echo "Устройство, откуда будет производиться копирование: ${ROOT_DEVICE}"
echo "Доступные устройства для которых можно произвести копирование: ${EMMC_DEVICES}"

# Преобразование списка доступных устройств в массив
EMMC_DEVICES_ARRAY=($EMMC_DEVICES)

# Проверка на пустоту массива устройств
if [ ${#EMMC_DEVICES_ARRAY[@]} -eq 0 ]; then
    whiptail \
      --title "Невозможно произвести копирование" \
      --msgbox "Не найдено ни одного устройства eMMC." \
      8 45
    exit 0
fi

# Проверка типа ОС и определение пути к загрузчику
echo "Определение текущей ОС..."
if [ -f /usr/local/repka-os-version ]; then
    echo "ОС определена: Repka OS"
    BOOT_MESSAGE=$(repka-control get-boot-message)
    if [ "${BOOT_MESSAGE}" = "0" ]; then
        echo "Режим загрузчика: тихий режим"
        UBOOT_PATH=/boot/spl/u-boot-sunxi-with-spl-silent.bin
    else
        echo "Режим загрузчика: стандартный режим"
        UBOOT_PATH=/boot/spl/u-boot-sunxi-with-spl.bin
    fi

elif [ -f /boot/dietpi/.version ]; then
    echo "ОС определена: DietPi"
    echo "Режим загрузчика: стандартный (DietPi)"
    UBOOT_PATH=/usr/lib/linux-u-boot-current-repka-pi4-optimal/u-boot-sunxi-with-spl.bin

elif [ -f /etc/os-release ] && grep -q 'NAME="Kali GNU/Linux"' /etc/os-release; then
    echo "ОС определена: Kali Linux"
    echo "Режим загрузчика: стандартный (Kali)"
    UBOOT_PATH=/usr/lib/u-boot/repka-pi4/u-boot-sunxi-with-spl.bin

elif [ -f /etc/os-release ] && grep -q 'NAME="Debian GNU/Linux"' /etc/os-release; then
    echo "ОС определена: Debian"
    echo "Режим загрузчика: стандартный (Debian)"
    UBOOT_PATH=/boot/spl/u-boot-sunxi-with-spl.bin

else
    whiptail \
      --title "Ошибка при определении ОС" \
      --msgbox "Утилита nand-sata-install не смогла определить ОС и путь, по-которому располагается бинарный файл загрузчика UBoot. Клонирование ОС - невозможно." \
      10 60
    exit 1
fi
echo "Путь к загрузчику: ${UBOOT_PATH}"

# Создание строки для whiptail
MENU_ITEMS=()
for DEVICE in "${EMMC_DEVICES_ARRAY[@]}"; do
    MENU_ITEMS+=("$DEVICE" "")
done

# Отображение меню с помощью whiptail
EMMC_DEVICE=$(
  whiptail \
    --title "Выбор устройства" \
    --menu "Копирование будет производиться из ${ROOT_DEVICE}\nВыберите устройство для копирования:" \
    16 60 4 \
    "${MENU_ITEMS[@]}" \
    3>&1 1>&2 2>&3)

# Проверка кода возврата
EXIT_STATUS=$?
if [ $EXIT_STATUS -eq 1 ] || [ $EXIT_STATUS -eq 255 ]; then
  exit 0
fi

# С какого сектора будет создан раздел на eMMC (взят безопасный отступ в 16 МБ)
FIRSTSECTOR=32768
# Размер в байтах реально используемого пространства на текущем root ("/")
USED_SIZE=$(df --output=used -B1 / | tail -n1 | xargs)
# Резерв в байтах, состоящий из первого сектора
RESERVE_BYTES=$((${FIRSTSECTOR} * 512))
# Общий размер данных в байтах, которые нужно уместить
REQUIRED_SIZE=$((USED_SIZE + RESERVE_BYTES))
# Размер eMMC в байтах
EMMC_SIZE=$(lsblk -bno SIZE "${EMMC_DEVICE}" | tail -1 | xargs)

# Проверка, возможно ли склонировать текущий root ("/") на eMMC
if [ "$EMMC_SIZE" -lt "$REQUIRED_SIZE" ]; then
    whiptail \
      --title "Ошибка" \
      --msgbox "Размер устройства eMMC (${EMMC_SIZE} байт) меньше, чем уже занятое пространство на ROOT-устройстве (${REQUIRED_SIZE} байт)." \
      8 45
    exit 0
fi

# Полное форматирование (безвозвратное) eMMC
echo "Полное форматирование (безвозвратное) eMMC (${EMMC_DEVICE})..."
blkdiscard ${EMMC_DEVICE}

# Удаление таблицы разделов на eMMC
echo "Удаление таблицы разделов на eMMC (${EMMC_DEVICE})..."
dd bs=1 seek=446 count=64 if=/dev/zero of="${EMMC_DEVICE}" >/dev/null 2>&1

# Подсчет размера eMMC
echo "Подсчет размера eMMC..."
QUOTED_DEVICE=$(echo "${EMMC_DEVICE}" | sed 's:/:\\\/:g')
CAPACITY=$(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", \$2 / ( 1024 / \$4 ))}")
echo "Размер eMMC: ~${CAPACITY} kB"

# Подсчет размера основного раздела на eMMC
# Если размер eMMC меньше 4 ГБ, то последний сектор будет 98% от общего объема
# Если размер eMMC больше 4 ГБ, то последний сектор будет 99% от общего объема
if [[ $CAPACITY -lt 4000000 ]]; then
		LASTSECTOR=$(( 32768 * $(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", ( \$2 * 98 / 3276800))}") -1 ))
	else
		LASTSECTOR=$(( 32768 * $(parted "${EMMC_DEVICE}" unit s print -sm | awk -F":" "/^${QUOTED_DEVICE}/ {printf (\"%0d\", ( \$2 * 99 / 3276800))}") -1 ))
fi
echo "Первый сектор на eMMC: ${FIRSTSECTOR}"
echo "Последний сектор на eMMC: ${LASTSECTOR}"

PART_TABLE_TYPE=$(parted "${ROOT_DEVICE}" print -sm | awk -F ":" -v pattern="${ROOT_DEVICE}" '$0 ~ pattern {print $6}')
echo "Устройство ${ROOT_DEVICE} имеет тип разметки диска: ${PART_TABLE_TYPE}"

parted -s "${EMMC_DEVICE}" -- mklabel "$PART_TABLE_TYPE"
parted -s "${EMMC_DEVICE}" -- mkpart primary ext4 ${FIRSTSECTOR}s ${LASTSECTOR}s
partprobe "${EMMC_DEVICE}"
mkfs.ext4 -qF "${EMMC_DEVICE}"'p1'
emmcuuid=$(blkid -o export "${EMMC_DEVICE}"'p1' | grep -w UUID)
emmcbootuuid=$emmcuuid
echo "Идентификатор eMMC: ${emmcuuid}"

echo "Запись загрузчика на eMMC..."
dd if=/dev/zero of=${EMMC_DEVICE} bs=1k count=1023 seek=1 status=noxfer > /dev/null 2>&1
dd if=${UBOOT_PATH} of=${EMMC_DEVICE} bs=1024 seek=8 status=noxfer > /dev/null 2>&1

echo "Создание точек монтирования..."
mkdir /mnt/root /mnt/emmc
mount "${ROOT_DEVICE}p1" /mnt/root
mount "${EMMC_DEVICE}p1" /mnt/emmc

echo "Копирование содержимого корневой файловой системы..."
rsync -aAXv --info=progress2 --exclude='/mnt/*' --exclude='/run/*' --exclude='/tmp/*' --exclude='/var/tmp/*' --exclude='/var/run/*' --exclude='/sys/*' --exclude='/proc/*' --exclude='/dev/*' /mnt/root/ /mnt/emmc/

# В случае, если ОС Kali Linux, необходимо изменить конфигурацию extlinux
if [ -f /etc/os-release ] && grep -q 'NAME="Kali GNU/Linux"' /etc/os-release; then
  sed -i 's|/dev/mmcblk0p1|/dev/mmcblk1p1|g' /mnt/emmc/boot/extlinux/extlinux.conf
fi

echo "Удаление точек монтирования и их содержимого..."
umount /mnt/root /mnt/emmc
rmdir /mnt/root /mnt/emmc

whiptail --title "OS Copying" --msgbox "Произошло удачное клонирование Repka OS с ${ROOT_DEVICE} на ${EMMC_DEVICE}" 8 45